/**
	Program to apply a filter in the frequency domain
	using the FFT functions 

	Fraga. Jun 12, 2001
**/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "matrix.h"
#include "tiff.h"
#include "fft.h"

int do_work( char *namein, int filtertype, int r ) 
{
	MATRIX *IN;
	MATRIX *FFTIMG;
	FILEINFO finfo;
	int i, j, signo;
	byte *iptr;
	float *fptr;
	

	IN = read_tiff( &finfo, namein ); 
	if ( IN==NULL ) {
		fprintf( stderr, "ERROR reading %s file\n", namein );
		return -1;
	}
	FFTIMG = matrix_allocate( IN->rows, IN->cols + 2, sizeof(float) );

	/** We copy the readed image to the new float matrix **/
	/** The variable signo is for move the FT a half of a period **/
	signo = -1;
	for ( i=0; i<IN->rows; i++ ) {
		iptr = (byte *)IN->ptr[i];
		fptr = (float *)FFTIMG->ptr[i];
		for ( j=0; j<IN->cols; j++ ) {
			signo *= -1;
			*fptr++ = *iptr++ * signo;
	//		*fptr++ = *iptr++;
		}
		signo *= -1;
	}

	/** Now we calculate the FFT **/
	image_fft ( (float **)FFTIMG->ptr, IN->rows , IN->cols, DIRECT );

	/** We apply the filter **/
	filter( (float **)FFTIMG->ptr, IN->rows , IN->cols, r, filtertype );

	/** For testing, we apply the inverse transform **/
	image_fft ( (float **)FFTIMG->ptr, IN->rows , IN->cols, INVERSE );

	/** Reverse the signs **/
	signo = -1;
	for ( i=0; i<IN->rows; i++ ) {
		fptr = (float *)FFTIMG->ptr[i];
		for ( j=0; j<IN->cols; j++ ) {
			signo *= -1;
			*fptr++ *= signo;
		}
		signo *= -1;
	}

	/** We put the result in a 8 bit matrix and then we store
		the resulted image. This must be the equal to the original image **/
	for ( i=0; i<IN->rows; i++ ){
		iptr = (byte *)IN->ptr[i];
		fptr = (float *)FFTIMG->ptr[i];
		for ( j=0; j<IN->rows; j++ ){ 
			*iptr++ = (*fptr) < 0.0 ? 0 : (*fptr) > 255 ? 255 : *fptr;
			fptr++;
		}
	}
	if ( write_tiff( IN, "filtered.tif" ) != GOOD_WRITE )
		return -1;

	matrix_free( IN );
	matrix_free( FFTIMG );
}

int main( int argc, char *argv[] )
{
	char namein[32];
	int type, radius;

	if ( argc < 4 ) { 
		fprintf( stderr, "Use: %s image.tif type radius\n", argv[0] );
		fprintf( stderr, "\t type=0 Lowpass, type=1 Highpass\n", argv[0] );
		return -1;
	}

	strncpy( namein, argv[1], 32 );
	type = atoi( argv[2] );
	radius = atoi( argv[3] );

	if ( do_work( namein, type, radius ) >= 0 ) {
		fprintf( stderr, "OK!!!\n" );
		return 0;
	}
	else {
		fprintf( stderr, "ERROR: Something is wrong!!!\n" );
		exit(1);
	}
	return 0;
}
