/** 
	Program to produce a High Boots filter

	Fraga Jun, 18 2001
**/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "matrix.h"
#include "tiff.h"

void normalization( MATRIX *IN )
{
	int i, j, imin, imax;
	byte *ptr;
	float scale=0.0;
	byte s[256];

	imin = 1000;
	imax = -1000;
	for( i=0; i<IN->rows; i++ ) {
		ptr = (byte *)IN->ptr[i];
		for( j=0; j<IN->cols; j++ ) {
			imin = (imin > ptr[j]) ? ptr[j] : imin;
			imax = (imax < ptr[j]) ? ptr[j] : imax;
		}
	}
	if ( imin != imax ) scale = 255.0/(imax-imin);
        for ( i=imin; i<=imax; i++ ) {
                s[i] = (byte)(scale*(i-imin) + 0.5 );
        }

	for( i=0; i<IN->rows; i++ ) {
		ptr = (byte *)IN->ptr[i];
		for( j=0; j<IN->cols; j++ ) 
			*ptr = s[ *ptr++ ];
	}
}

MATRIX *convolutionf( MATRIX *img, MATRIX *win, int kw )
{
	MATRIX *OUT;
	int i, j, k, l, s;
	float sum, val;
	byte *ptrin, *ptrout;
	float *ptrw;

	OUT = matrix_allocate( img->rows, img->cols, sizeof(byte) );
	if ( OUT==NULL ) {
		fprintf( stderr, "ERROR allocating memory for output image\n" );
		return NULL;
	}
	for ( i=0; i < img->rows; i++ ) {
		ptrout = (byte *) OUT->ptr[i];
		for ( j=0; j < img->cols; j++ ) 
			*ptrout++ = 0;
	}

	/** Suponemos que la ventana es cuadrada y de taqmao impar **/
	k = win -> rows / 2;

	for ( i=0; i <= img->rows - win->rows; i++ ) {
		ptrout = (byte *) OUT->ptr[i + k] + k;

		for ( j=0; j <= img->cols - win->cols; j++ ) {
			sum = 0;
			for ( l=0; l < win->rows; l++ ) {
				ptrin = (byte *)img->ptr[ i + l ] + j;
				ptrw = (float *)win->ptr[ l ];
				for ( s=0; s < win->cols; s++ ) 
					sum += *ptrin++ * *ptrw++;
			}
			val = (int)( (1.0/kw)*sum + 0.5 );
			*ptrout++ = (val<0) ? 0 : (val>255) ? 255 : (byte)val;
		}
	}

	return OUT;	
}

int do_work( char *namein, float Avalue ) 
{
	MATRIX *IN, *OUT, *FIL;
	FILEINFO finfo;
	float val;
	float **fil;
	

	IN = read_tiff( &finfo, namein ); 
	if ( IN==NULL ) {
		fprintf( stderr, "ERROR reading %s file\n", namein );
		return -1;
	}
	if ( IN->type != 5 ) {
		fprintf( stderr, "ERROR Bad format. Image must be in 256 grey tones\n");
		return -1;
	}
	FIL = matrix_allocate( 3, 3, sizeof( float ) );
	if ( FIL==NULL ) {
		fprintf( stderr, "ERROR allocating memory\n" );
		return -1;
	}
	fil = (float **)FIL->ptr;
	val = 9*Avalue - 1.0;
	fil[0][0] =-1.0; fil[0][1] =-1.0; fil[0][2] =-1.0;
	fil[1][0] =-1.0; fil[1][1] = val; fil[1][2] =-1.0;
	fil[2][0] =-1.0; fil[2][1] =-1.0; fil[2][2] =-1.0;

	OUT = convolutionf( IN, FIL, 9 );
	normalization( OUT );
	if ( OUT == NULL ) {
		fprintf( stderr, "ERROR doing convolution\n");
	 	return -1;
	}
	OUT->type=5;
	if ( write_tiff( OUT, "hboost.tif" ) != GOOD_WRITE )  
        return -1;

	matrix_free( OUT );
	matrix_free( FIL );
	matrix_free( IN );

	return 0;
}

int main( int argc, char *argv[] )
{
	char namein[32];
	float aval;

	if ( argc < 3 ) { 
		fprintf( stderr, "Use: %s image.tif A_value\n", argv[0] );
		return -1;
	}

	strncpy( namein, argv[1], 32 );

	aval = atof( argv[2] );

	if ( aval < 1.0 ) {
		fprintf( stderr, "A must be greater than 1\n" );
		exit(1);
	}
	if ( do_work( namein, aval ) >= 0 ) {
		fprintf( stderr, "OK!!!\n" );
		return 0;
	}
	else {
		fprintf( stderr, "ERROR: Something is wrong!!!\n" );
		exit(1);
	}
	return 0;
}
