/**
	Program to calculate the histogram of a image 

	Input image must be in TIFF format

	Fraga. Jun 4, 2001
**/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "matrix.h"
#include "tiff.h"
#include "fft.h"

int do_work( char *namein ) 
{
	MATRIX *IN;
	MATRIX *HIST;
	FILEINFO finfo;
	int i, j;
	byte *iptr;
	int *histo;
	

	IN = read_tiff( &finfo, namein ); 
	if ( IN==NULL ) {
		fprintf( stderr, "ERROR reading %s file\n", namein );
		return -1;
	}
	if ( IN->type != 5 ) {
		fprintf( stderr, "ERROR Bad format. Image must be in 256 grey tones\n");
		return -1;
	}

	HIST = matrix_allocate( 1, 256, sizeof(int) );
	if ( HIST==NULL ) {
		fprintf( stderr, "ERROR allocating memory\n" );
		return -1;
	}
	histo = (int *)HIST->ptr[0];

	for ( i=0; i<256; i++ ) histo[i] = 0;

	/** We copy the readed image to the new float matrix **/
	/** The variable signo is for move the FT a half of a period **/
	for ( i=0; i<IN->rows; i++ ) {
		iptr = (byte *)IN->ptr[i];
		for ( j=0; j<IN->cols; j++ ) { 
			histo[ *iptr ] ++;
			iptr++;
		}
	}
	for ( i=0; i<256; i++ )
		printf( "%d %d\n", i,  histo[i] );

	matrix_free( IN );
	matrix_free( HIST );
}

int main( int argc, char *argv[] )
{
	char namein[32];

	if ( argc < 2 ) { 
		fprintf( stderr, "Use: %s image.tif\n", argv[0] );
		return -1;
	}

	strncpy( namein, argv[1], 32 );

	if ( do_work( namein ) >= 0 ) {
		fprintf( stderr, "OK!!!\n" );
		return 0;
	}
	else {
		fprintf( stderr, "ERROR: Something is wrong!!!\n" );
		exit(1);
	}
	return 0;
}
