#!/usr/bin/perl

# Perl program to construct a image mask to
# be used for testing the compresions methos with
# the Discrete Cosine Transform (DCT)

# Output: mask in .... format
# Input: value of n and radius
#
# Fraga July 11, 2001

die "Args: mask_size radius\n" if ( $#ARGV != 1 );

$n = $ARGV[0];
$r = $ARGV[1];
# print "$n $r\n";

for ( $i=0; $i<$n; $i++ ) {
	for ( $j=0; $j<$n; $j++ ) {
		$img[ $n * $i + $j ] = 1;
	}
}
$R2 = $r * $r;
$n_coef = 0;
for ( $i=0; $i<$n; $i++ ) {
	$r1 = $i * $i;
	for ( $j=0; $j<$n; $j++ ) {
		$r2 = $j * $j;
		$f = $r1 + $r2 - $R2;
		if ( $f <= 0 ) { 
			$img[ $n * $i + $j ] = 0;
			$n_coef++;
		}
	}
}
$percent = $n_coef / ( $n * $n );
printf "Number of coefficients = $n_coef\n";
printf "Percent of coeffcients = $percent\n";
$percent = $n * $n;
print "# Total number of coeffcients = $percent\n";

print "Mask in file \"mask.pbm\"\n";

$val = save_pbm_img ( \@img, $n, $n, "mask.pbm" );
if ( !$val ) {
	printf "All's ok!\n";
}
else {
	print "You have problems...\n";
}

save_c ( \@img, $n, $n, "mask.h" );

# Subroutine to save a image in ASCII PBM format
# (see pbm man pages on NETPBM package)
sub save_pbm_img (\@$$$) {  # ( @img, $rows, $cols );
	local (*myimg) = shift(@_);
	my $myrows = shift(@_);
	my $mycols = shift(@_);
	my $filename = shift(@_);

	my $n = @myimg;
	# printf "$n $mycols $myrows $filename\n";

	open ( MYFILE, ">$filename" ) or return 1;

	print MYFILE "P1\n";
	print MYFILE "# file: $filename\n";
	print MYFILE "$mycols $myrows\n";

	for ( my $i=1; $i <= $myrows*$mycols; $i++ ) {
		printf( MYFILE "%2d ", $myimg[$i-1] ); 
		if ( !($i%30) ) {
			print MYFILE "\n";
		}
	}
	close MYFILE;
	return 0;
}

sub save_c (\@$$$) {  # ( @img, $rows, $cols );
	local (*myimg) = shift(@_);
	my $myrows = shift(@_);
	my $mycols = shift(@_);
	my $filename = shift(@_);

	my $n = @myimg;
	# printf "$n $mycols $myrows $filename\n";

	open ( MYFILE, ">$filename" ) or return 1;

	print MYFILE "#define mask_width $mycols\n";
	print MYFILE "#define mask_height $myrows\n";
	print MYFILE "static char mask_bits[$mycols][$myrows] = {\n";

	for ( my $i=0; $i < $myrows; $i++ ) {
		print MYFILE "\t{ ";
		for ( my $j=1; $j < $myrows; $j++ ) {
			printf( MYFILE "%d, ", $myimg[$mycols* $i + $j] ); 
		}
		print MYFILE "},\n";
	}
	print MYFILE "};\n";
	close MYFILE;
	return 0;
}

